@@ -1,23 +1,27 @@
 /* This code is does a SCSI READ CAPACITY command on the given device
-   and outputs the result.
-
-*  Copyright (C) 1999 - 2015 D. Gilbert
-*  This program is free software; you can redistribute it and/or modify
-*  it under the terms of the GNU General Public License as published by
-*  the Free Software Foundation; either version 2, or (at your option)
-*  any later version.
-
-   This program was originally written with Linux 2.4 kernel series.
-   It now builds for the Linux 2.6 and 3 kernel series and various other
-   operating systems.
-
-*/
+ * and outputs the result.
+ *
+ * Copyright (C) 1999 - 2020 D. Gilbert
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
+ * This program was originally written with Linux 2.4 kernel series.
+ * It now builds for the Linux 2.6, 3 and 4 kernel series and various other
+ * operating systems.
+ */
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdarg.h>
+#include <stdbool.h>
 #include <unistd.h>
 #include <string.h>
 #include <fcntl.h>
+#include <errno.h>
 #include <getopt.h>
 #define __STDC_FORMAT_MACROS 1
 #include <inttypes.h>
@@ -25,13 +29,14 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+
 #include "sg_lib.h"
 #include "sg_cmds_basic.h"
 #include "sg_unaligned.h"
 #include "sg_pr2serr.h"
 
 
-static const char * version_str = "3.95 20151219";
+static const char * version_str = "4.05 20200122";
 
 #define ME "sg_readcap: "
 
@@ -39,50 +44,53 @@ static const char * version_str = "3.95 20151219";
 #define RCAP16_REPLY_LEN 32
 
 static struct option long_options[] = {
-    {"brief", 0, 0, 'b'},
-    {"help", 0, 0, 'h'},
-    {"hex", 0, 0, 'H'},
-    {"lba", 1, 0, 'L'},
-    {"long", 0, 0, 'l'},
-    {"16", 0, 0, 'l'},
-    {"new", 0, 0, 'N'},
-    {"old", 0, 0, 'O'},
-    {"pmi", 0, 0, 'p'},
-    {"raw", 0, 0, 'r'},
-    {"readonly", 0, 0, 'R'},
-    {"verbose", 0, 0, 'v'},
-    {"version", 0, 0, 'V'},
-    {"zbc", 0, 0, 'z'},
+    {"brief", no_argument, 0, 'b'},
+    {"help", no_argument, 0, 'h'},
+    {"hex", no_argument, 0, 'H'},
+    {"lba", required_argument, 0, 'L'},
+    {"long", no_argument, 0, 'l'},
+    {"16", no_argument, 0, 'l'},
+    {"new", no_argument, 0, 'N'},
+    {"old", no_argument, 0, 'O'},
+    {"pmi", no_argument, 0, 'p'},
+    {"raw", no_argument, 0, 'r'},
+    {"readonly", no_argument, 0, 'R'},
+    {"verbose", no_argument, 0, 'v'},
+    {"version", no_argument, 0, 'V'},
+    {"zbc", no_argument, 0, 'z'},
     {0, 0, 0, 0},
 };
 
 struct opts_t {
-    int do_brief;
+    bool do_brief;
+    bool do_long;
+    bool do_pmi;
+    bool do_raw;
+    bool o_readonly;
+    bool do_zbc;
+    bool opt_new;
+    bool verbose_given;
+    bool version_given;
     int do_help;
     int do_hex;
     int do_lba;
-    int do_long;
-    int do_pmi;
-    int do_raw;
-    int o_readonly;
-    int do_verbose;
-    int do_version;
-    int do_zbc;
+    int verbose;
     uint64_t llba;
     const char * device_name;
-    int opt_new;
 };
 
 
 static void
 usage()
 {
-    pr2serr("Usage: sg_readcap [--brief] [--help] [--hex] [--lba=LBA] "
-            "[--long] [--16]\n"
+    pr2serr("Usage: sg_readcap [--16] [--brief] [--help] [--hex] "
+            "[--lba=LBA] [--long]\n"
             "                  [--pmi] [--raw] [--readonly] [--verbose] "
             "[--version]\n"
             "                  [--zbc] DEVICE\n"
             "  where:\n"
+            "    --16            use READ CAPACITY (16) cdb (same as "
+            "--long)\n"
             "    --brief|-b      brief, two hex numbers: number of blocks "
             "and block size\n"
             "    --help|-h       print this usage message and exit\n"
@@ -93,8 +101,6 @@ usage()
             "valid with '--pmi']\n"
             "    --long|-l       use READ CAPACITY (16) cdb (def: use "
             "10 byte cdb)\n"
-            "    --16            use READ CAPACITY (16) cdb (same as "
-            "--long)\n"
             "    --pmi|-p        partial medium indicator (without this "
             "option shows\n"
             "                    total disk capacity) [made obsolete in "
@@ -104,6 +110,7 @@ usage()
             "read-write)\n"
             "    --verbose|-v    increase verbosity\n"
             "    --version|-V    print version string and exit\n"
+            "    --old|-O        use old interface (use as first option)\n"
             "    --zbc|-z        show rc_basis ZBC field (implies --16)\n\n"
             "Perform a SCSI READ CAPACITY (10 or 16) command\n");
 }
@@ -132,6 +139,7 @@ usage_old()
             "    -R     open DEVICE read-only (def: RCAP(16) read-write)\n"
             "    -v     increase verbosity\n"
             "    -V     print version string and exit\n"
+            "    -N|--new   use new interface\n"
             "    -z     show rc_basis ZBC field (implies -16)\n\n"
             "Perform a SCSI READ CAPACITY (10 or 16) command\n");
 }
@@ -146,7 +154,7 @@ usage_for(const struct opts_t * op)
 }
 
 static int
-process_cl_new(struct opts_t * op, int argc, char * argv[])
+new_parse_cmd_line(struct opts_t * op, int argc, char * argv[])
 {
     int c;
     int a_one = 0;
@@ -166,10 +174,10 @@ process_cl_new(struct opts_t * op, int argc, char * argv[])
             break;
         case '6':
             if (a_one)
-                ++op->do_long;
+                op->do_long = true;
             break;
         case 'b':
-            ++op->do_brief;
+            op->do_brief = true;
             break;
         case 'h':
         case '?':
@@ -179,7 +187,7 @@ process_cl_new(struct opts_t * op, int argc, char * argv[])
             ++op->do_hex;
             break;
         case 'l':
-            ++op->do_long;
+            op->do_long = true;
             break;
         case 'L':
             nn = sg_get_llnum(optarg);
@@ -191,31 +199,32 @@ process_cl_new(struct opts_t * op, int argc, char * argv[])
             op->llba = nn;
             /* force READ_CAPACITY16 for large lbas */
             if (op->llba > 0xfffffffeULL)
-                ++op->do_long;
+                op->do_long = true;
             ++op->do_lba;
             break;
         case 'N':
             break;      /* ignore */
         case 'O':
-            op->opt_new = 0;
+            op->opt_new = false;
             return 0;
         case 'p':
-            ++op->do_pmi;
+            op->do_pmi = true;
             break;
         case 'r':
-            ++op->do_raw;
+            op->do_raw = true;
             break;
         case 'R':
-            ++op->o_readonly;
+            op->o_readonly = true;
             break;
         case 'v':
-            ++op->do_verbose;
+            op->verbose_given = true;
+            ++op->verbose;
             break;
         case 'V':
-            ++op->do_version;
+            op->version_given = true;
             break;
         case 'z':
-            ++op->do_zbc;
+            op->do_zbc = true;
             break;
         default:
             pr2serr("unrecognised option code %c [0x%x]\n", c, c);
@@ -241,9 +250,10 @@ process_cl_new(struct opts_t * op, int argc, char * argv[])
 }
 
 static int
-process_cl_old(struct opts_t * op, int argc, char * argv[])
+old_parse_cmd_line(struct opts_t * op, int argc, char * argv[])
 {
-    int k, jmp_out, plen, num;
+    bool jmp_out;
+    int k, plen, num;
     const char * cp;
     uint64_t uu;
 
@@ -253,18 +263,18 @@ process_cl_old(struct opts_t * op, int argc, char * argv[])
         if (plen <= 0)
             continue;
         if ('-' == *cp) {
-            for (--plen, ++cp, jmp_out = 0; plen > 0; --plen, ++cp) {
+            for (--plen, ++cp, jmp_out = false; plen > 0; --plen, ++cp) {
                 switch (*cp) {
                 case '1':
                     if ('6' == *(cp + 1)) {
-                        ++op->do_long;
+                        op->do_long = true;
                         ++cp;
                         --plen;
                     } else
-                        jmp_out = 1;
+                        jmp_out = true;
                     break;
                 case 'b':
-                    ++op->do_brief;
+                    op->do_brief = true;
                     break;
                 case 'h':
                 case '?':
@@ -274,35 +284,36 @@ process_cl_old(struct opts_t * op, int argc, char * argv[])
                     ++op->do_hex;
                     break;
                 case 'N':
-                    op->opt_new = 1;
+                    op->opt_new = true;
                     return 0;
                 case 'O':
                     break;
                 case 'p':
                     if (0 == strncmp("pmi", cp, 3)) {
-                        ++op->do_pmi;
+                        op->do_pmi = true;
                         cp += 2;
                         plen -= 2;
                     } else
-                        jmp_out = 1;
+                        jmp_out = true;
                     break;
                 case 'r':
-                    ++op->do_raw;
+                    op->do_raw = true;
                     break;
                 case 'R':
-                    ++op->o_readonly;
+                    op->o_readonly = true;
                     break;
                 case 'v':
-                    ++op->do_verbose;
+                    op->verbose_given = true;
+                    ++op->verbose;
                     break;
                 case 'V':
-                    ++op->do_version;
+                    op->version_given = true;
                     break;
                 case 'z':
-                    ++op->do_zbc;
+                    op->do_zbc = true;
                     break;
                 default:
-                    jmp_out = 1;
+                    jmp_out = true;
                     break;
                 }
                 if (jmp_out)
@@ -319,7 +330,7 @@ process_cl_old(struct opts_t * op, int argc, char * argv[])
                 }
                 /* force READ_CAPACITY16 for large lbas */
                 if (uu > 0xfffffffeULL)
-                    ++op->do_long;
+                    op->do_long = true;
                 op->llba = uu;
                 ++op->do_lba;
             } else if (0 == strncmp("-old", cp, 4))
@@ -342,32 +353,32 @@ process_cl_old(struct opts_t * op, int argc, char * argv[])
 }
 
 static int
-process_cl(struct opts_t * op, int argc, char * argv[])
+parse_cmd_line(struct opts_t * op, int argc, char * argv[])
 {
     int res;
     char * cp;
 
     cp = getenv("SG3_UTILS_OLD_OPTS");
     if (cp) {
-        op->opt_new = 0;
-        res = process_cl_old(op, argc, argv);
+        op->opt_new = false;
+        res = old_parse_cmd_line(op, argc, argv);
         if ((0 == res) && op->opt_new)
-            res = process_cl_new(op, argc, argv);
+            res = new_parse_cmd_line(op, argc, argv);
     } else {
-        op->opt_new = 1;
-        res = process_cl_new(op, argc, argv);
-        if ((0 == res) && (0 == op->opt_new))
-            res = process_cl_old(op, argc, argv);
+        op->opt_new = true;
+        res = new_parse_cmd_line(op, argc, argv);
+        if ((0 == res) && (! op->opt_new))
+            res = old_parse_cmd_line(op, argc, argv);
     }
     return res;
 }
 
 static void
-dStrRaw(const char* str, int len)
+dStrRaw(const uint8_t * str, int len)
 {
     int k;
 
-    for (k = 0 ; k < len; ++k)
+    for (k = 0; k < len; ++k)
         printf("%c", str[k]);
 }
 
@@ -392,31 +403,51 @@ rc_basis_str(int rc_basis, char * b, int blen)
 int
 main(int argc, char * argv[])
 {
-    int sg_fd, res, prot_en, p_type, lbppbe, rw_0_flag;
-    uint64_t llast_blk_addr;
+    bool rw_0_flag;
+    int res, prot_en, p_type, lbppbe;
+    int sg_fd = -1;
     int ret = 0;
     uint32_t last_blk_addr, block_size;
-    unsigned char resp_buff[RCAP16_REPLY_LEN];
+    uint64_t llast_blk_addr;
+    uint8_t * resp_buff;
+    uint8_t * free_resp_buff;
+    const int resp_buff_sz = RCAP16_REPLY_LEN;
     char b[80];
     struct opts_t opts;
     struct opts_t * op;
 
     op = &opts;
     memset(op, 0, sizeof(opts));
-    res = process_cl(op, argc, argv);
+    res = parse_cmd_line(op, argc, argv);
     if (res)
-        return SG_LIB_SYNTAX_ERROR;
+        return res;
     if (op->do_help) {
         usage_for(op);
         return 0;
     }
-    if (op->do_version) {
+#ifdef DEBUG
+    pr2serr("In DEBUG mode, ");
+    if (op->verbose_given && op->version_given) {
+        pr2serr("but override: '-vV' given, zero verbose and continue\n");
+        op->verbose_given = false;
+        op->version_given = false;
+        op->verbose = 0;
+    } else if (! op->verbose_given) {
+        pr2serr("set '-vv'\n");
+        op->verbose = 2;
+    } else
+        pr2serr("keep verbose=%d\n", op->verbose);
+#else
+    if (op->verbose_given && op->version_given)
+        pr2serr("Not in DEBUG mode, so '-vV' has no special action\n");
+#endif
+    if (op->version_given) {
         pr2serr("Version string: %s\n", version_str);
         return 0;
     }
 
     if (NULL == op->device_name) {
-        pr2serr("No DEVICE argument given\n");
+        pr2serr("No DEVICE argument given\n\n");
         usage_for(op);
         return SG_LIB_SYNTAX_ERROR;
     }
@@ -428,40 +459,46 @@ main(int argc, char * argv[])
     }
     if (op->do_zbc) {
         if (! op->do_long)
-            ++op->do_long;
+            op->do_long = true;
     }
 
-    memset(resp_buff, 0, sizeof(resp_buff));
-
-    if ((0 == op->do_pmi) && (op->llba > 0)) {
+    resp_buff = sg_memalign(resp_buff_sz, 0, &free_resp_buff, false);
+    if (NULL == resp_buff) {
+        pr2serr("Unable to allocate %d bytes on heap\n", resp_buff_sz);
+        return sg_convert_errno(ENOMEM);
+    }
+    if ((! op->do_pmi) && (op->llba > 0)) {
         pr2serr(ME "lba can only be non-zero when '--pmi' is set\n");
         usage_for(op);
-        return SG_LIB_SYNTAX_ERROR;
+        ret = SG_LIB_CONTRADICT;
+        goto fini;
     }
     if (op->do_long)
         rw_0_flag = op->o_readonly;
     else
-        rw_0_flag = 1;  /* RCAP(10) has opened RO in past, so leave */
+        rw_0_flag = true;  /* RCAP(10) has opened RO in past, so leave */
     if ((sg_fd = sg_cmds_open_device(op->device_name, rw_0_flag,
-                                     op->do_verbose)) < 0) {
+                                     op->verbose)) < 0) {
         pr2serr(ME "error opening file: %s: %s\n", op->device_name,
                 safe_strerror(-sg_fd));
-        return SG_LIB_FILE_ERROR;
+        ret = sg_convert_errno(-sg_fd);
+        goto fini;
     }
 
     if (! op->do_long) {
         res = sg_ll_readcap_10(sg_fd, op->do_pmi, (unsigned int)op->llba,
-                               resp_buff, RCAP_REPLY_LEN, 1, op->do_verbose);
+                               resp_buff, RCAP_REPLY_LEN, true,
+                               op->verbose);
         ret = res;
         if (0 == res) {
             if (op->do_hex || op->do_raw) {
                 if (op->do_raw)
-                    dStrRaw((const char *)resp_buff, RCAP_REPLY_LEN);
+                    dStrRaw(resp_buff, RCAP_REPLY_LEN);
                 else if (op->do_hex > 2)
-                    dStrHex((const char *)resp_buff, RCAP_REPLY_LEN, -1);
+                    hex2stdout(resp_buff, RCAP_REPLY_LEN, -1);
                 else
-                    dStrHex((const char *)resp_buff, RCAP_REPLY_LEN, 1);
-                goto good;
+                    hex2stdout(resp_buff, RCAP_REPLY_LEN, 1);
+                goto fini;
             }
             last_blk_addr = sg_get_unaligned_be32(resp_buff + 0);
             if (0xffffffff != last_blk_addr) {
@@ -469,7 +506,7 @@ main(int argc, char * argv[])
                 if (op->do_brief) {
                     printf("0x%" PRIx32 " 0x%" PRIx32 "\n",
                            last_blk_addr + 1, block_size);
-                    goto good;
+                    goto fini;
                 }
                 printf("Read Capacity results:\n");
                 if (op->do_pmi)
@@ -477,9 +514,9 @@ main(int argc, char * argv[])
                            "before delay=0x%" PRIx32 "\n", op->llba,
                            last_blk_addr);
                 else
-                    printf("   Last logical block address=%" PRIu32 " (0x%"
-                           PRIx32 "), Number of blocks=%" PRIu32 "\n",
-                           last_blk_addr, last_blk_addr, last_blk_addr + 1);
+                    printf("   Last LBA=%" PRIu32 " (0x%" PRIx32 "), Number "
+                           "of logical blocks=%" PRIu32 "\n", last_blk_addr,
+                            last_blk_addr, last_blk_addr + 1);
                 printf("   Logical block length=%u bytes\n", block_size);
                 if (! op->do_pmi) {
                     uint64_t total_sz = last_blk_addr + 1;
@@ -493,55 +530,64 @@ main(int argc, char * argv[])
                     printf("Hence:\n");
 #ifdef SG_LIB_MINGW
                     printf("   Device size: %" PRIu64 " bytes, %g MiB, %g "
-                           "GB\n", total_sz, sz_mb, sz_gb);
+                           "GB", total_sz, sz_mb, sz_gb);
 #else
                     printf("   Device size: %" PRIu64 " bytes, %.1f MiB, "
-                           "%.2f GB\n", total_sz, sz_mb, sz_gb);
+                           "%.2f GB", total_sz, sz_mb, sz_gb);
 #endif
+                    if (sz_gb > 2000) {
+#ifdef SG_LIB_MINGW
+                        printf(", %g TB", sz_gb / 1000);
+#else
+                        printf(", %.2f TB", sz_gb / 1000);
+#endif
+                    }
+                    printf("\n");
                 }
-                goto good;
+                goto fini;
             } else {
                 printf("READ CAPACITY (10) indicates device capacity too "
                        "large\n  now trying 16 byte cdb variant\n");
-                op->do_long = 1;
+                op->do_long = true;
             }
         } else if (SG_LIB_CAT_INVALID_OP == res) {
-            op->do_long = 1;
+            op->do_long = true;
             sg_cmds_close_device(sg_fd);
             if ((sg_fd = sg_cmds_open_device(op->device_name, op->o_readonly,
-                                             op->do_verbose)) < 0) {
+                                             op->verbose)) < 0) {
                 pr2serr(ME "error re-opening file: %s (rw): %s\n",
                         op->device_name, safe_strerror(-sg_fd));
-                return SG_LIB_FILE_ERROR;
+                ret = sg_convert_errno(-sg_fd);
+                goto fini;
             }
-            if (op->do_verbose)
+            if (op->verbose)
                 pr2serr("READ CAPACITY (10) not supported, trying READ "
                         "CAPACITY (16)\n");
         } else if (res) {
-            sg_get_category_sense_str(res, sizeof(b), b, op->do_verbose);
+            sg_get_category_sense_str(res, sizeof(b), b, op->verbose);
             pr2serr("READ CAPACITY (10) failed: %s\n", b);
         }
     }
     if (op->do_long) {
         res = sg_ll_readcap_16(sg_fd, op->do_pmi, op->llba, resp_buff,
-                               RCAP16_REPLY_LEN, 1, op->do_verbose);
+                               RCAP16_REPLY_LEN, true, op->verbose);
         ret = res;
         if (0 == res) {
             if (op->do_hex || op->do_raw) {
                 if (op->do_raw)
-                    dStrRaw((const char *)resp_buff, RCAP16_REPLY_LEN);
+                    dStrRaw(resp_buff, RCAP16_REPLY_LEN);
                 else if (op->do_hex > 2)
-                    dStrHex((const char *)resp_buff, RCAP16_REPLY_LEN, -1);
+                    hex2stdout(resp_buff, RCAP16_REPLY_LEN, -1);
                 else
-                    dStrHex((const char *)resp_buff, RCAP16_REPLY_LEN, 1);
-                goto good;
+                    hex2stdout(resp_buff, RCAP16_REPLY_LEN, 1);
+                goto fini;
             }
             llast_blk_addr = sg_get_unaligned_be64(resp_buff + 0);
             block_size = sg_get_unaligned_be32(resp_buff + 8);
             if (op->do_brief) {
                 printf("0x%" PRIx64 " 0x%" PRIx32 "\n", llast_blk_addr + 1,
                        block_size);
-                goto good;
+                goto fini;
             }
             prot_en = !!(resp_buff[12] & 0x1);
             p_type = ((resp_buff[12] >> 1) & 0x7);
@@ -565,9 +611,9 @@ main(int argc, char * argv[])
                        "before delay=0x%" PRIx64 "\n", op->llba,
                        llast_blk_addr);
             else
-                printf("   Last logical block address=%" PRIu64 " (0x%"
-                       PRIx64 "), Number of logical blocks=%" PRIu64 "\n",
-                       llast_blk_addr, llast_blk_addr, llast_blk_addr + 1);
+                printf("   Last LBA=%" PRIu64 " (0x%" PRIx64 "), Number of "
+                       "logical blocks=%" PRIu64 "\n", llast_blk_addr,
+                       llast_blk_addr, llast_blk_addr + 1);
             printf("   Logical block length=%" PRIu32 " bytes\n", block_size);
             lbppbe = resp_buff[13] & 0xf;
             printf("   Logical blocks per physical block exponent=%d",
@@ -577,7 +623,7 @@ main(int argc, char * argv[])
                        block_size * (1 << lbppbe));
             else
                 printf("\n");
-            printf("   Lowest aligned logical block address=%d\n",
+            printf("   Lowest aligned LBA=%d\n",
                    ((resp_buff[14] & 0x3f) << 8) + resp_buff[15]);
             if (! op->do_pmi) {
                 uint64_t total_sz = llast_blk_addr + 1;
@@ -590,31 +636,47 @@ main(int argc, char * argv[])
                         (double)(1000000000L);
                 printf("Hence:\n");
 #ifdef SG_LIB_MINGW
-                printf("   Device size: %" PRIu64 " bytes, %g MiB, %g GB\n",
+                printf("   Device size: %" PRIu64 " bytes, %g MiB, %g GB",
                        total_sz, sz_mb, sz_gb);
 #else
                 printf("   Device size: %" PRIu64 " bytes, %.1f MiB, %.2f "
-                       "GB\n", total_sz, sz_mb, sz_gb);
+                       "GB", total_sz, sz_mb, sz_gb);
 #endif
+                if (sz_gb > 2000) {
+#ifdef SG_LIB_MINGW
+                    printf(", %g TB", sz_gb / 1000);
+#else
+                    printf(", %.2f TB", sz_gb / 1000);
+#endif
+                }
+                printf("\n");
             }
-            goto good;
+            goto fini;
         } else if (SG_LIB_CAT_ILLEGAL_REQ == res)
             pr2serr("bad field in READ CAPACITY (16) cdb including "
                     "unsupported service action\n");
         else if (res) {
-            sg_get_category_sense_str(res, sizeof(b), b, op->do_verbose);
+            sg_get_category_sense_str(res, sizeof(b), b, op->verbose);
             pr2serr("READ CAPACITY (16) failed: %s\n", b);
         }
     }
     if (op->do_brief)
         printf("0x0 0x0\n");
-
-good:
-    res = sg_cmds_close_device(sg_fd);
-    if (res < 0) {
-        pr2serr("close error: %s\n", safe_strerror(-res));
-        if (0 == ret)
-            return SG_LIB_FILE_ERROR;
+fini:
+    if (free_resp_buff)
+        free(free_resp_buff);
+    if (sg_fd >= 0) {
+        res = sg_cmds_close_device(sg_fd);
+        if (res < 0) {
+            pr2serr("close error: %s\n", safe_strerror(-res));
+            if (0 == ret)
+                ret = sg_convert_errno(-res);
+        }
+    }
+    if (0 == op->verbose) {
+        if (! sg_if_can2stderr("sg_readcap failed: ", ret))
+            pr2serr("Some error occurred, try again with '-v' "
+                    "or '-vv' for more information\n");
     }
     return (ret >= 0) ? ret : SG_LIB_CAT_OTHER;
 }
