@@ -1,8 +1,10 @@
 /*
- * Copyright (c) 2014-2018 Douglas Gilbert.
+ * Copyright (c) 2014-2019 Douglas Gilbert.
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
+ *
+ * SPDX-License-Identifier: BSD-2-Clause
  */
 
 #include <unistd.h>
@@ -26,6 +28,7 @@
 #endif
 
 #include "sg_lib.h"
+#include "sg_lib_data.h"
 #include "sg_cmds_basic.h"
 #include "sg_cmds_extra.h"
 #include "sg_unaligned.h"
@@ -42,7 +45,7 @@
  * RESULTS commands in order to send microcode to the given SES device.
  */
 
-static const char * version_str = "1.16 20180628";    /* ses4r02 */
+static const char * version_str = "1.18 20190513";    /* ses4r02 */
 
 #define ME "sg_ses_microcode: "
 #define MAX_XFER_LEN (128 * 1024 * 1024)
@@ -118,13 +121,10 @@ static struct mode_s mode_arr[] = {
     {NULL, 0, NULL},
 };
 
-struct diag_page_code {
-    int page_code;
-    const char * desc;
-};
-
-/* An array of Download microcode status field values and descriptions */
-static struct diag_page_code mc_status_arr[] = {
+/* An array of Download microcode status field values and descriptions.
+ * This table is a subset of one in sg_read_buffer for the read microcode
+ * status page. */
+static struct sg_lib_simple_value_name_t mc_status_arr[] = {
     {0x0, "No download microcode operation in progress"},
     {0x1, "Download in progress, awaiting more"},
     {0x2, "Download complete, updating storage"},
@@ -239,11 +239,11 @@ print_modes(void)
 static const char *
 get_mc_status_str(uint8_t status_val)
 {
-    const struct diag_page_code * mcsp;
+    const struct sg_lib_simple_value_name_t * mcsp;
 
-    for (mcsp = mc_status_arr; mcsp->desc; ++mcsp) {
-        if (status_val == mcsp->page_code)
-            return mcsp->desc;
+    for (mcsp = mc_status_arr; mcsp->name; ++mcsp) {
+        if (status_val == mcsp->value)
+            return mcsp->name;
     }
     return "";
 }
@@ -933,7 +933,7 @@ fini:
         }
     }
     if (0 == op->verbose) {
-        if (! sg_if_can2stderr("sg_ses_mocrocode failed: ", ret))
+        if (! sg_if_can2stderr("sg_ses_microcode failed: ", ret))
             pr2serr("Some error occurred, try again with '-v' "
                     "or '-vv' for more information\n");
     }
