@@ -1,8 +1,10 @@
 /*
- * Copyright (c) 2004-2018 Douglas Gilbert.
+ * Copyright (c) 2004-2019 Douglas Gilbert.
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
+ *
+ * SPDX-License-Identifier: BSD-2-Clause
  */
 
 #include <unistd.h>
@@ -35,7 +37,7 @@
  * (e.g. disks).
  */
 
-static const char * version_str = "1.23 20180628";
+static const char * version_str = "1.25 20191220";
 
 #define SYNCHRONIZE_CACHE16_CMD     0x91
 #define SYNCHRONIZE_CACHE16_CMDLEN  16
@@ -95,7 +97,7 @@ sg_ll_sync_cache_16(int sg_fd, bool sync_nv, bool immed, int group,
                     uint64_t lba, unsigned int num_lb, int to_secs,
                     bool noisy, int verbose)
 {
-    int res, ret, k, sense_cat;
+    int res, ret, sense_cat;
     uint8_t sc_cdb[SYNCHRONIZE_CACHE16_CMDLEN] =
                 {SYNCHRONIZE_CACHE16_CMD, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                  0, 0, 0, 0, 0, 0};
@@ -111,10 +113,11 @@ sg_ll_sync_cache_16(int sg_fd, bool sync_nv, bool immed, int group,
     sg_put_unaligned_be32((uint32_t)num_lb, sc_cdb + 10);
 
     if (verbose) {
-        pr2serr("    synchronize cache(16) cdb: ");
-        for (k = 0; k < SYNCHRONIZE_CACHE16_CMDLEN; ++k)
-            pr2serr("%02x ", sc_cdb[k]);
-        pr2serr("\n");
+        char b[128];
+
+        pr2serr("    Synchronize cache(16) cdb: %s\n",
+                sg_get_command_str(sc_cdb, SYNCHRONIZE_CACHE16_CMDLEN, false,
+                                   sizeof(b), b));
     }
     ptvp = construct_scsi_pt_obj();
     if (NULL == ptvp) {
@@ -125,8 +128,7 @@ sg_ll_sync_cache_16(int sg_fd, bool sync_nv, bool immed, int group,
     set_scsi_pt_sense(ptvp, sense_b, sizeof(sense_b));
     res = do_scsi_pt(ptvp, sg_fd, to_secs, verbose);
     ret = sg_cmds_process_resp(ptvp, "synchronize cache(16)", res,
-                               SG_NO_DATA_IN, sense_b, noisy, verbose,
-                               &sense_cat);
+                               noisy, verbose, &sense_cat);
     if (-1 == ret)
         ret = sg_convert_errno(get_scsi_pt_os_err(ptvp));
     else if (-2 == ret) {
