@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2019 Douglas Gilbert
+ * Copyright (c) 2014-2020 Douglas Gilbert
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
@@ -40,7 +40,7 @@
 #include "sg_unaligned.h"
 #include "sg_pr2serr.h"
 
-static const char * version_str = "1.17 20191220";
+static const char * version_str = "1.18 20200430";
 
 
 #define ME "sg_write_verify: "
@@ -175,6 +175,10 @@ run_scsi_transaction(int sg_fd, const uint8_t *cdbp, int cdb_len,
             }
             ret = sense_cat;
             break;
+        case SG_LIB_CAT_ILLEGAL_REQ:
+            if (verbose)
+                sg_print_command_len(cdbp, cdb_len);
+            /* FALL THROUGH */
         case SG_LIB_CAT_PROTECTION:     /* PI failure */
         case SG_LIB_CAT_MISCOMPARE:     /* only in bytchk=1 case */
         default:
@@ -242,7 +246,7 @@ sg_ll_write_verify16(int sg_fd, int wrprotect, bool dpo, int bytchk,
     return ret;
 }
 
-/* Returns file descriptor ( >= 0) if successfule. Else a negated sg3_utils
+/* Returns file descriptor ( >= 0) if successful. Else a negated sg3_utils
  * error code is returned. */
 static int
 open_if(const char * fn, int got_stdin)
