@@ -1,7 +1,7 @@
 /* A utility program for copying files. Specialised for "files" that
  * represent devices that understand the SCSI command set.
  *
- * Copyright (C) 1999 - 2020 D. Gilbert and P. Allworth
+ * Copyright (C) 1999 - 2021 D. Gilbert and P. Allworth
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
@@ -22,7 +22,7 @@
  * in this case) are transferred to or from the sg device in a single SCSI
  * command.
  *
- * This version is designed for the linux kernel 2.4, 2.6, 3 and 4 series.
+ * This version is designed for the linux kernel 2.4, 2.6, 3, 4 and 5 series.
  *
  * sgp_dd is a Posix threads specialization of the sg_dd utility. Both
  * sgp_dd and sg_dd only perform special tasks when one or both of the given
@@ -84,7 +84,7 @@
 #include "sg_pr2serr.h"
 
 
-static const char * version_str = "5.75 20200219";
+static const char * version_str = "5.77 20210103";
 
 #define DEF_BLOCK_SIZE 512
 #define DEF_BLOCKS_PER_TRANSFER 128
@@ -192,7 +192,7 @@ typedef struct request_element
     uint8_t * buffp;
     uint8_t * alloc_bp;
     struct sg_io_hdr io_hdr;
-    uint8_t cmd[MAX_SCSI_CDBSZ];
+    uint8_t cdb[MAX_SCSI_CDBSZ];
     uint8_t sb[SENSE_BUFF_LEN];
     int bs;
     int dio_incomplete_count;
@@ -229,14 +229,17 @@ static atomic_uint ascending_val;
 static pthread_mutex_t av_mut = PTHREAD_MUTEX_INITIALIZER;
 static int ascending_val = 1;
 
-#define GET_NEXT_PACK_ID(_v)                            \
-    ( { int _r;                                         \
-    do {                                                \
-        pthread_mutex_lock(&av_mut);                    \
-        _r = ascending_val;                             \
-        ascending_val += _v;                            \
-        pthread_mutex_lock(&av_mut);                    \
-    } while (0) ; _r; } )
+static unsigned int
+GET_NEXT_PACK_ID(unsigned int val)
+{
+    int res;
+
+    pthread_mutex_lock(&av_mut);
+    res = ascending_val;
+    ascending_val += val;
+    pthread_mutex_unlock(&av_mut);
+    return res;
+}
 
 #endif
 
@@ -370,7 +373,7 @@ tsafe_strerror(int code, char * ebp)
  * ISBN 0-201-63392-2 . [Highly recommended book.] Changed __FILE__
  * to __func__ */
 #define err_exit(code,text) do { \
-    char strerr_buff[STRERR_BUFF_LEN]; \
+    char strerr_buff[STRERR_BUFF_LEN + 1]; \
     pr2serr("%s at \"%s\":%d: %s\n", \
         text, __func__, __LINE__, tsafe_strerror(code, strerr_buff)); \
     exit(1); \
@@ -689,18 +692,15 @@ sg_out_open(const char * fnp, struct flags_t * flagp, int bs, int bpt)
 static void *
 read_write_thread(void * v_tap)
 {
-    Thread_arg * tap;
-    Rq_coll * clp;
+    Thread_arg * tap = (Thread_arg *)v_tap;
+    Rq_coll * clp = tap->clp;
     Rq_elem rel;
     Rq_elem * rep = &rel;
     int sz;
     volatile bool stop_after_write = false;
     int64_t seek_skip;
-    int blocks, status, id;
+    int blocks, status;
 
-    tap = (Thread_arg *)v_tap;
-    id = tap->id;
-    clp = tap->clp;
     sz = clp->bpt * clp->bs;
     seek_skip =  clp->seek - clp->skip;
     memset(rep, 0, sizeof(Rq_elem));
@@ -732,7 +732,6 @@ read_write_thread(void * v_tap)
     if (clp->mmap_active) {
         int fd = clp->in_flags.mmap ? rep->infd : rep->outfd;
 
-pr2serr("%s: id=%d, fd=%d calling sgp_mem_mmap()\n", __func__, id, fd);
         status = sgp_mem_mmap(fd, sz, &rep->buffp);
         if (status) err_exit(status, "sgp_mem_mmap() failed");
     } else {
@@ -840,7 +839,7 @@ normal_in_operation(Rq_coll * clp, Rq_elem * rep, int blocks)
 {
     bool stop_after_write = false;
     int res;
-    char strerr_buff[STRERR_BUFF_LEN];
+    char strerr_buff[STRERR_BUFF_LEN + 1];
 
     /* enters holding in_mutex */
     while (((res = read(rep->infd, rep->buffp, blocks * clp->bs)) < 0) &&
@@ -886,7 +885,7 @@ static void
 normal_out_operation(Rq_coll * clp, Rq_elem * rep, int blocks)
 {
     int res;
-    char strerr_buff[STRERR_BUFF_LEN];
+    char strerr_buff[STRERR_BUFF_LEN + 1];
 
     /* enters holding out_mutex */
     while (((res = write(rep->outfd, rep->buffp, rep->num_blks * clp->bs))
@@ -1054,6 +1053,10 @@ sg_in_operation(Rq_coll * clp, Rq_elem * rep)
             status = pthread_mutex_unlock(&clp->in_mutex);
             if (0 != status) err_exit(status, "unlock in_mutex");
             return;
+        case SG_LIB_CAT_ILLEGAL_REQ:
+            if (clp->debug)
+                sg_print_command_len(rep->cdb, rep->cdbsz_in);
+            /* FALL THROUGH */
         default:
             pr2serr("error finishing sg in command (%d)\n", res);
             if (exit_status <= 0)
@@ -1128,6 +1131,10 @@ sg_out_operation(Rq_coll * clp, Rq_elem * rep)
             status = pthread_mutex_unlock(&clp->out_mutex);
             if (0 != status) err_exit(status, "unlock out_mutex");
             return;
+        case SG_LIB_CAT_ILLEGAL_REQ:
+            if (clp->debug)
+                sg_print_command_len(rep->cdb, rep->cdbsz_out);
+            /* FALL THROUGH */
         default:
             pr2serr("error finishing sg out command (%d)\n", res);
             if (exit_status <= 0)
@@ -1149,7 +1156,7 @@ sg_start_io(Rq_elem * rep)
     int cdbsz = rep->wr ? rep->cdbsz_out : rep->cdbsz_in;
     int res;
 
-    if (sg_build_scsi_cdb(rep->cmd, cdbsz, rep->num_blks, rep->blk,
+    if (sg_build_scsi_cdb(rep->cdb, cdbsz, rep->num_blks, rep->blk,
                           rep->wr, fua, dpo)) {
         pr2serr("%sbad cdb build, start_blk=%" PRId64 ", blocks=%d\n",
                 my_name, rep->blk, rep->num_blks);
@@ -1158,7 +1165,7 @@ sg_start_io(Rq_elem * rep)
     memset(hp, 0, sizeof(struct sg_io_hdr));
     hp->interface_id = 'S';
     hp->cmd_len = cdbsz;
-    hp->cmdp = rep->cmd;
+    hp->cmdp = rep->cdb;
     hp->dxfer_direction = rep->wr ? SG_DXFER_TO_DEV : SG_DXFER_FROM_DEV;
     hp->dxfer_len = rep->bs * rep->num_blks;
     hp->dxferp = mmap ? NULL : rep->buffp;
