@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2018-2019 D. Gilbert
+ *  Copyright (C) 2018-2020 D. Gilbert
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
@@ -60,14 +60,14 @@
  * later of the Linux sg driver.  */
 
 
-static const char * version_str = "Version: 1.14  20191116";
+static const char * version_str = "Version: 1.19  20200825";
 
 #define INQ_REPLY_LEN 128
 #define INQ_CMD_LEN 6
 #define SDIAG_CMD_LEN 6
 #define SENSE_BUFFER_LEN 96
 
-#define EBUFF_SZ 256
+#define EBUFF_SZ 512
 
 #ifndef SG_FLAG_Q_AT_TAIL
 #define SG_FLAG_Q_AT_TAIL 0x10
@@ -97,30 +97,43 @@ static bool show_size_value = false;
 static bool do_v3_only = false;
 
 static int childs_pid = 0;
+static int iterator_test = -1;
+static int object_walk_test = -1;
 static int sg_drv_ver_num = 0;
 static int q_len = DEF_Q_LEN;
 static int sleep_secs = 0;
 static int reserve_buff_sz = DEF_RESERVE_BUFF_SZ;
 static int num_mrqs = 0;
 static int num_sgnw = 0;
+static int dname_current = 0;
+static int dname_last = 0;
+static int dname_pos = 0;
 static int verbose = 0;
 
-static const char * relative_cp = NULL;
+static const char * relative_cp = "";
 static char * file_name = NULL;
 
 
 static void
 usage(void)
 {
-    printf("Usage: sg_tst_ioctl [-3] [-c] [-f] [-h] [-l=Q_LEN] "
-           "[-m=MRQS[,I|S]]\n"
-           "                    [-M] [-n] [-o] [-r=SZ] [-s=SEC] [-S] [-t] "
-           "[-T=NUM]\n"
-           "                    [-v] [-V] [-w] <sg_device> [<sg_device2>]\n"
+    printf("Usage: sg_tst_ioctl [-3] [-c] [-f] [-h] [-I=0|1] [-J=0|1] "
+           "[-l=Q_LEN]\n"
+           "                    [-m=MRQS[,I|S]] [-M] [-n] [-o] [-r=SZ] "
+           "[-s=SEC]\n"
+           "                    [-S] [-t] [-T=NUM] [-v] [-V] [-w]\n"
+           "                    <sg_device>[-<num>] [<sg_device2>]\n"
            " where:\n"
+           "      -3      use sg v3 interface (def: sg v4 if available)\n"
            "      -c      timestamp when sg driver created <sg_device>\n"
            "      -f      fork and test share between processes\n"
            "      -h      help: print usage message then exit\n"
+           "      -I=0|1    iterator test of mid-level; 0: unlocked, 1: "
+           "locked\n"
+           "                does test -T=NUM times, outputs duration\n"
+           "      -J=0|1    object walk up then 2 lookups; 0: no logging; "
+           "1: log\n"
+           "                up-scan once per 1000 iterations\n"
            "      -l=Q_LEN    queue length, between 1 and 511 (def: 16)\n"
            "      -m=MRQS[,I|S]    test multi-req, MRQS number to do; if "
            "the letter\n"
@@ -143,7 +156,14 @@ usage(void)
            "                ioctl(SG_GET_NUM_WAITING); then exit\n"
            "      -v    increase verbosity of output\n"
            "      -V    print version string then exit\n"
-           "      -w    write (submit) only then exit\n");
+           "      -w    write (submit) only then exit\n\n");
+    printf("There are various groups of options for different tests. The "
+           "get_num_waiting\ngroup needs '-T=NUM' given. When '-I=0|1' is "
+           "also given then an object tree\niterator test is done NUM "
+           "times. If instead '-J=0|1' is given then an\nobject tree "
+           "traversal (up/down) is done 10,000 times (and NUM is\n"
+           "ignored).\n"
+          );
 }
 
 static void
@@ -402,10 +422,10 @@ tst_extended_ioctl(const char * fnp, int sg_fd, const char * fn2p, int sg_fd2,
     seip->ctl_flags_rd_mask |= SG_CTL_FLAGM_ORPHANS;
     seip->ctl_flags_rd_mask |= SG_CTL_FLAGM_Q_TAIL;
     seip->ctl_flags_rd_mask |= SG_CTL_FLAGM_IS_SHARE;
-    seip->ctl_flags_rd_mask |= SG_CTL_FLAGM_IS_MASTER;
+    seip->ctl_flags_rd_mask |= SG_CTL_FLAGM_IS_READ_SIDE;
     seip->ctl_flags_rd_mask |= SG_CTL_FLAGM_UNSHARE;
-    seip->ctl_flags_rd_mask |= SG_CTL_FLAGM_MASTER_FINI;
-    seip->ctl_flags_rd_mask |= SG_CTL_FLAGM_MASTER_ERR;
+    seip->ctl_flags_rd_mask |= SG_CTL_FLAGM_READ_SIDE_FINI;
+    seip->ctl_flags_rd_mask |= SG_CTL_FLAGM_READ_SIDE_ERR;
     seip->ctl_flags_wr_mask |= SG_CTL_FLAGM_NO_DURATION;
     seip->ctl_flags_rd_mask |= SG_CTL_FLAGM_NO_DURATION;
     seip->ctl_flags |= SG_CTL_FLAGM_TIME_IN_NS;
@@ -442,18 +462,18 @@ tst_extended_ioctl(const char * fnp, int sg_fd, const char * fn2p, int sg_fd2,
         if (SG_CTL_FLAGM_IS_SHARE & seip->ctl_flags_rd_mask)
             printf("  %sIS_SHARE: %s\n", cp,
                    (SG_CTL_FLAGM_IS_SHARE & cflags) ? "true" : "false");
-        if (SG_CTL_FLAGM_IS_MASTER & seip->ctl_flags_rd_mask)
-            printf("  %sIS_MASTER: %s\n", cp,
-                   (SG_CTL_FLAGM_IS_MASTER & cflags) ? "true" : "false");
+        if (SG_CTL_FLAGM_IS_READ_SIDE & seip->ctl_flags_rd_mask)
+            printf("  %sIS_READ_SIDE: %s\n", cp,
+                   (SG_CTL_FLAGM_IS_READ_SIDE & cflags) ? "true" : "false");
         if (SG_CTL_FLAGM_UNSHARE & seip->ctl_flags_rd_mask)
             printf("  %sUNSHARE: %s\n", cp,
                    (SG_CTL_FLAGM_UNSHARE & cflags) ? "true" : "false");
-        if (SG_CTL_FLAGM_MASTER_FINI & seip->ctl_flags_rd_mask)
-            printf("  %sMASTER_FINI: %s\n", cp,
-                   (SG_CTL_FLAGM_MASTER_FINI & cflags) ? "true" : "false");
-        if (SG_CTL_FLAGM_MASTER_ERR & seip->ctl_flags_rd_mask)
-            printf("  %sMASTER_ERR: %s\n", cp,
-                   (SG_CTL_FLAGM_MASTER_ERR & cflags) ? "true" : "false");
+        if (SG_CTL_FLAGM_READ_SIDE_FINI & seip->ctl_flags_rd_mask)
+            printf("  %sREAD_SIDE_FINI: %s\n", cp,
+                   (SG_CTL_FLAGM_READ_SIDE_FINI & cflags) ? "true" : "false");
+        if (SG_CTL_FLAGM_READ_SIDE_ERR & seip->ctl_flags_rd_mask)
+            printf("  %sREAD_SIDE_ERR: %s\n", cp,
+                   (SG_CTL_FLAGM_READ_SIDE_ERR & cflags) ? "true" : "false");
         if (SG_CTL_FLAGM_NO_DURATION & seip->ctl_flags_rd_mask)
             printf("  %sNO_DURATION: %s\n", cp,
                    (SG_CTL_FLAGM_NO_DURATION & cflags) ? "true" : "false");
@@ -645,7 +665,7 @@ do_mrqs(int sg_fd, int sg_fd2, int mrqs)
     mrq_h4p->flags = SGV4_FLAG_MULTIPLE_REQS;
     if (mrq_immed)
         mrq_h4p->flags |= SGV4_FLAG_IMMED;
-    arr_v4 = calloc(mrqs, sizeof(struct sg_io_v4));
+    arr_v4 = (struct sg_io_v4 *)calloc(mrqs, sizeof(struct sg_io_v4));
     if (NULL == arr_v4) {
         res = ENOMEM;
         goto fini;
@@ -748,10 +768,13 @@ fini:
 int
 main(int argc, char * argv[])
 {
-    bool done;
+    bool done, is_first;
     bool nw_given = false;
-    int sg_fd, k, ok, pack_id, num_waiting;
+    bool has_dname_range = false;
+    int k, ok, pack_id, num_waiting;
     int res = 0;
+    int sum_nw = 0;
+    int sg_fd = -1;
     int sg_fd2 = -1;
     int sock = -1;
     uint8_t inq_cdb[INQ_CMD_LEN] =
@@ -762,9 +785,11 @@ main(int argc, char * argv[])
     sg_io_hdr_t io_hdr[MAX_Q_LEN];
     sg_io_hdr_t rio_hdr;
     char ebuff[EBUFF_SZ];
+    char dname[256];
     uint8_t sense_buffer[MAX_Q_LEN][SENSE_BUFFER_LEN];
     const char * second_fname = NULL;
     const char * cp;
+    char * chp;
     struct sg_scsi_id ssi;
 
 
@@ -781,6 +806,20 @@ main(int argc, char * argv[])
         else if (0 == memcmp("-h", argv[k], 2)) {
             file_name = 0;
             break;
+        } else if (0 == memcmp("-I=", argv[k], 3)) {
+            iterator_test = atoi(argv[k] + 3);
+            if ((iterator_test > 1) || (iterator_test < -1)) {
+                printf("Expect -I= to take a number, either 0 or 1\n");
+                file_name = 0;
+                break;
+            }
+        } else if (0 == memcmp("-J=", argv[k], 3)) {
+            object_walk_test = atoi(argv[k] + 3);
+            if ((object_walk_test > 1) || (object_walk_test < -1)) {
+                printf("Expect -J= to take a number, either 0 or 1\n");
+                file_name = 0;
+                break;
+            }
         } else if (0 == memcmp("-l=", argv[k], 3)) {
             q_len = atoi(argv[k] + 3);
             if ((q_len > 511) || (q_len < 1)) {
@@ -878,6 +917,9 @@ main(int argc, char * argv[])
             break;
         }
     }
+    if ((iterator_test >= 0) || (object_walk_test >= 0))
+        nw_given = false;
+
     if (show_size_value) {
         struct utsname unam;
 
@@ -928,46 +970,156 @@ main(int argc, char * argv[])
         usage();
         return 1;
     }
+    memset(dname, 0, sizeof(dname));
+    if (strlen(file_name) > 255) {
+        fprintf(stderr, "file_name too long\n");
+        goto out;
+    }
+    strncpy(dname, file_name, sizeof(dname) - 1);
+    if ((chp = strchr(dname, '-'))) {
+        if (1 != sscanf(chp + 1, "%d", &dname_last)) {
+            fprintf(stderr, "can't code number after '-' in file_name\n");
+            goto out;
+        }
+        *chp = '\0';
+        --chp;
+        while (isdigit(*chp))
+            --chp;
+        ++chp;
+        if (1 != sscanf(chp, "%d", &dname_current)) {
+            fprintf(stderr, "can't code number before '-' in file_name\n");
+            goto out;
+        }
+        *chp = '\0';
+        has_dname_range = true;
+        dname_pos = strlen(dname);
+    }
+    is_first = true;
+
+dname_range_loop:
+    if (has_dname_range)
+        sprintf(dname + dname_pos, "%d", dname_current);
 
     /* An access mode of O_RDWR is required for write()/read() interface */
-    if ((sg_fd = open(file_name, O_RDWR)) < 0) {
-        snprintf(ebuff, EBUFF_SZ,
-                 "error opening file: %s", file_name);
+    if ((sg_fd = open(dname, O_RDWR)) < 0) {
+        snprintf(ebuff, EBUFF_SZ, "error opening file: %s", dname);
         perror(ebuff);
         return 1;
     }
     if (verbose)
         fprintf(stderr, "opened given file: %s successfully, fd=%d\n",
-                file_name, sg_fd);
+                dname, sg_fd);
 
     if (ioctl(sg_fd, SG_GET_VERSION_NUM, &sg_drv_ver_num) < 0) {
         pr2serr("ioctl(SG_GET_VERSION_NUM) failed, errno=%d %s\n", errno,
                 strerror(errno));
         goto out;
     }
-    printf("Linux sg driver version: %d\n", sg_drv_ver_num);
+    if (is_first)
+        printf("Linux sg driver version: %d\n", sg_drv_ver_num);
 
     if (create_time && (sg_drv_ver_num > 40030)) {
-        pr_create_dev_time(sg_fd, file_name);
+        pr_create_dev_time(sg_fd, dname);
         goto out;
     }
 
-    if (nw_given) {             /* time ioctl(SG_GET_NUM_WAITING) */
-        int nw, sum_nw;
+    if (nw_given || (iterator_test >= 0) || (object_walk_test >= 0)) {
+        /* -T=NUM and/or -I=0|1 or -j=0|1 */
+        /* time ioctl(SG_GET_NUM_WAITING) or do iterator_test */
+        int nw;
         struct timespec start_tm, fin_tm, res_tm;
 
-        printf("Timing %d calls to ioctl(SG_GET_NUM_WAITING)\n", num_sgnw);
-        if (0 != clock_gettime(CLOCK_MONOTONIC, &start_tm)) {
-                res = errno;
-                perror("start clock_gettime() failed:");
-                goto out;
+        if (is_first) {
+            int rang = has_dname_range ? (1 + dname_last - dname_current) : 1;
+
+            is_first = false;
+            if (nw_given)
+                printf("Timing %d x %d calls to ioctl(SG_GET_NUM_WAITING)\n",
+                       rang, num_sgnw);
+            else if (iterator_test >= 0) {
+                k = num_sgnw + 1000;
+                printf("Timing %d calls to ioctl(SG_SET_DEBUG, %d)\n",
+                       rang, ((0 == iterator_test) ? -k : k));
+            } else
+                printf("Timing %d calls to ioctl(SG_SET_DEBUG, %d)\n",
+                       rang, (object_walk_test == 0) ? 999 : -999);
+            if (0 != clock_gettime(CLOCK_MONOTONIC, &start_tm)) {
+                    res = errno;
+                    perror("start clock_gettime() failed:");
+                    goto out;
+            }
         }
-        for (k = 0, sum_nw = 0; k < num_sgnw; ++k, sum_nw += nw) {
-            if (ioctl(sg_fd, SG_GET_NUM_WAITING, &nw) < 0) {
+        if (nw_given) {
+            for (k = 0; k < num_sgnw; ++k, sum_nw += nw) {
+                if (ioctl(sg_fd, SG_GET_NUM_WAITING, &nw) < 0) {
+                    res = errno;
+                    fprintf(stderr, "%d: ioctl(SG_GET_NUM_WAITING) failed "
+                            "errno=%d\n", k, res);
+                    goto out;
+                }
+            }
+        } else if (iterator_test >= 0) {
+            int fd, pid;
+
+            k = num_sgnw + 1000;
+            if (0 == iterator_test)
+                k = -k;
+            if (second_fname) {
+                if ((sg_fd2 = open(second_fname, O_RDWR)) < 0) {
+                    snprintf(ebuff, EBUFF_SZ, "%s: error opening file: %s",
+                             __func__, second_fname);
+                    perror(ebuff);
+                    return 1;
+                }
+                printf("About to fork due to second filename\n");
+                pid = fork();
+                if (pid < 0) {
+                    perror("fork() failed");
+                    goto out;
+                } else if (0 == pid) {
+                    relative_cp = "child: ";
+                    is_parent = false;
+                    fd = sg_fd2;
+                } else {
+                    relative_cp = "parent: ";
+                    is_parent = true;
+                    childs_pid = pid;
+                    fd = sg_fd;
+                }
+            } else {
+                fd = sg_fd;
+                relative_cp = "";
+            }
+            if (ioctl(fd, SG_SET_DEBUG, &k) < 0) {
                 res = errno;
-                fprintf(stderr, "%d: ioctl(SG_GET_NUM_WAITING) failed "
-                        "errno=%d\n", k, res);
+                fprintf(stderr, "%s%d: ioctl(SG_SET_DEBUG) failed errno=%d\n",
+                        relative_cp, k, res);
                 goto out;
+            } else if (verbose)
+                fprintf(stderr, "%siterator_test good ioctl(SG_SET_DEBUG, "
+                        "%d)\n", relative_cp, k);
+            sum_nw += num_sgnw;
+        } else if (object_walk_test >= 0) {
+            const char * ccp = "object_walk_test";
+
+            relative_cp = "";
+            k = (object_walk_test == 0) ? 999 : -999;
+            if (ioctl(sg_fd, SG_SET_DEBUG, &k) < 0) {
+                res = errno;
+                fprintf(stderr, "%s: ioctl(SG_SET_DEBUG, %d) failed "
+                        "errno=%d\n", ccp, k, res);
+            } else if (verbose)
+                fprintf(stderr, "%s: good call to ioctl(SG_SET_DEBUG, %d)\n",
+                        ccp, k);
+            sum_nw += 10000;    /* (1_up-scan + 2_lookups) * 10,000 times */
+        }
+
+        if (has_dname_range) {
+            ++dname_current;
+            if (dname_current <= dname_last) {
+                if (sg_fd >= 0)
+                    close(sg_fd);
+                goto dname_range_loop;
             }
         }
         if (0 != clock_gettime(CLOCK_MONOTONIC, &fin_tm)) {
@@ -982,21 +1134,21 @@ main(int argc, char * argv[])
             res_tm.tv_nsec += 1000000000;
         }
         if (verbose) {
-            if (verbose > 1)
+            if (nw_given && (verbose > 1))
                 printf("sum of num_waiting_s=%d\n", sum_nw);
-            printf("elapsed time (nanosecond precision): %d.%09d secs\n",
-                   (int)res_tm.tv_sec, (int)res_tm.tv_nsec);
+            printf("%selapsed time (nanosecond precision): %d.%09d secs\n",
+                   relative_cp, (int)res_tm.tv_sec, (int)res_tm.tv_nsec);
         } else
-            printf("elapsed time: %d.%06d secs\n", (int)res_tm.tv_sec,
-                   (int)(res_tm.tv_nsec / 1000));
+            printf("%selapsed time: %d.%06d secs\n", relative_cp,
+                   (int)res_tm.tv_sec, (int)(res_tm.tv_nsec / 1000));
         if (num_sgnw >= 100) {
             double m = (double)res_tm.tv_sec +
                        ((double)res_tm.tv_nsec / 1000000000.0);
+            double num = num_sgnw;
 
             if (m > 0.000001)
-                printf("Calls per second: %.2f\n", (double)num_sgnw / m);
+                printf("%sCalls per second: %.2f\n", relative_cp, num / m);
         }
-
         res = 0;
         goto out;
     }
@@ -1052,7 +1204,7 @@ main(int argc, char * argv[])
 
     cp = do_fork ? relative_cp : "";
     if (! do_v3_only) {
-        if (tst_extended_ioctl(file_name, sg_fd, second_fname, sg_fd2, sock,
+        if (tst_extended_ioctl(dname, sg_fd, second_fname, sg_fd2, sock,
                                cp))
             goto out;
     }
@@ -1191,7 +1343,8 @@ main(int argc, char * argv[])
     }
 
 out:
-    close(sg_fd);
+    if (sg_fd >= 0)
+        close(sg_fd);
     if (sg_fd2 >= 0)
         close(sg_fd2);
     return res;
